Package: rgdal
Title: Bindings for the 'Geospatial' Data Abstraction Library
Version: 1.5-29
Date: 2022-03-07
Depends: R (>= 3.5.0), methods, sp (>= 1.1-0)
Imports: grDevices, graphics, stats, utils
LinkingTo: sp
Suggests: knitr, DBI, RSQLite, maptools, mapview, rmarkdown, curl,
        rgeos
NeedsCompilation: yes
Description: Provides bindings to the 'Geospatial' Data Abstraction Library ('GDAL') (>= 1.11.4) and access to projection/transformation operations from the 'PROJ' library. Please note that 'rgdal' will be retired by the end of 2023, plan transition to sf/stars/'terra' functions using 'GDAL' and 'PROJ' at your earliest convenience. Use is made of classes defined in the 'sp' package. Raster and vector map data can be imported into R, and raster and vector 'sp' objects exported. The 'GDAL' and 'PROJ' libraries are external to the package, and, when installing the package from source, must be correctly installed first; it is important that 'GDAL' < 3 be matched with 'PROJ' < 6. From 'rgdal' 1.5-8, installed with to 'GDAL' >=3, 'PROJ' >=6 and 'sp' >= 1.4, coordinate reference systems use 'WKT2_2019' strings, not 'PROJ' strings. 'Windows' and 'macOS' binaries (including 'GDAL', 'PROJ' and their dependencies) are provided on 'CRAN'. 
Authors@R: c(
	person("Roger", "Bivand", role = c("cre", "aut"), email = "Roger.Bivand@nhh.no", comment=c(ORCID="0000-0003-2392-6140")),
	person("Tim", "Keitt", role = "aut"),
	person("Barry", "Rowlingson", role = c("aut", "ctb")),
	person("Edzer", "Pebesma", role = "ctb"),
	person("Michael", "Sumner", role = "ctb"),
	person("Robert", "Hijmans", role = "ctb"),
        person("Daniel", "Baston", role = "ctb"),
	person("Even", "Rouault", role = c("cph", "ctb")),
	person("Frank", "Warmerdam", role = c("cph", "ctb")),
	person("Jeroen", "Ooms", role = "ctb"),
	person("Colin", "Rundel", role = "ctb"))
License: GPL (>= 2)
URL: http://rgdal.r-forge.r-project.org, https://gdal.org,
        https://proj.org, https://r-forge.r-project.org/projects/rgdal/
SystemRequirements: PROJ (>= 4.8.0, https://proj.org/download.html) and
        GDAL (>= 1.11.4, https://gdal.org/download.html), with versions
        either (A) PROJ < 6 and GDAL < 3 or (B) PROJ >= 6 and GDAL >=
        3. For degraded PROJ >= 6 and GDAL < 3, use the configure
        argument '--with-proj_api="proj_api.h"'.
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Packaged: 2022-03-15 08:11:38 UTC; rsb
Author: Roger Bivand [cre, aut] (<https://orcid.org/0000-0003-2392-6140>),
  Tim Keitt [aut],
  Barry Rowlingson [aut, ctb],
  Edzer Pebesma [ctb],
  Michael Sumner [ctb],
  Robert Hijmans [ctb],
  Daniel Baston [ctb],
  Even Rouault [cph, ctb],
  Frank Warmerdam [cph, ctb],
  Jeroen Ooms [ctb],
  Colin Rundel [ctb]
Maintainer: Roger Bivand <Roger.Bivand@nhh.no>
Repository: CRAN
Date/Publication: 2022-03-16 22:10:02 UTC
Built: R 4.0.5; x86_64-apple-darwin17.0; 2022-03-17 10:25:19 UTC; unix
Archs: rgdal.so.dSYM
